/***************************************************************************
                          block.cpp  -  description
                             -------------------
    begin                : Fri May 16 2003
    copyright            : (C) 2003 by Marcus Beranek
    email                : marcus@beranek.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// *************************************
// * How do we represent a protocol??? *
// *************************************
//
// Physically a protocol consists of streamed bits and bytes.
// But logically, we see those bits organized in blocks, every block having
// a different meaning from the other blocks.
//
// So a protocol is a sequence of logical-blocks. Unfortunatly, this sequence
// is not like a search-tree allways. It often has the structure of an unordered
// graph. For example in SMTP you can send a "NOOP"- or "RESET"-command during 
// the whole session. Theoretically your SMTP-session could be an infinite loop
// of those two commands, which (in the real world) makes no sense.
//
// To represent such blocks in a protocol, i am using two adjacency-lists in this
// class: fls and fby. The blocks, which this block follows, are stored in the
// fls-list (fls = follows), the blocks, which this block is followed by, are
// stored in the fby-list (fby = followedby).
//
// The entries in these two lists are of the "Successor"-Class. This class has
// only one member: the ID of the followed- or followedby-block.
//
// The list itself is derived from the QPtrList-class.
//
// Due to the fact, that some protocols may have different openings (e.g. SMTP:
// "HELO" or "EHLO"), every protocol gets a root-block with an ID = 0. So we
// have a well defined starting-block, end every opening-block of the protocol
// is stored in the fby-list of that starting-block.
//
// Additionally we have a third list named "typical". In this list we store a
// typical successor of our current block. For example in SMTP, after issuing 
// the "HELO"-command, it would be typical to issue the "MAIL"-command. So the
// "typical"-list is a hint, how the protocol should look and the "fls/fby"-
// lists are a hint, how a protocol could look like.
//
//
 
#include "block.h"
#include "successorlist.h"

Block::Block()
{
	init(0);
}
//---------------------------------------------------------------------------


Block::Block( int id )
{
	init( id );
}
//---------------------------------------------------------------------------


// Copy-Constructor:
//
// kann leider i.A. noch nicht so richtig verwendet werden, da eine QPtrList verwendet wird!
// Auf die kann hier nicht zugegriffen werden... :(
//
// Dies funktioniert zwar im Prinzip. Wenn wir jedoch den CC. aus einem
// Plugin heraus aufrufen, machen diese QPtrListen rger!
//
//
Block::Block( Block& b ) : QObject()
{
	ID = b.ID;
	Depth = b.Depth;
	visited = b.visited;
	printed = b.printed;
	Type = b.Type;
	Name = b.Name;
	State = b.State;
	Description = b.Description;
	Function = b.Function;
	Command = b.Command;
	Example = b.Example;
	fby = b.getFollowedby();               // <- hier Probleme...
	fls = b.getFollows();                  // <- hier Probleme...
	typical = b.getTypicalFollowedby();    // <- hier Probleme...
}
//---------------------------------------------------------------------------



Block::~Block()
{
}
//---------------------------------------------------------------------------


void Block::init( int id )
{
	ID = id;
	Depth = 0;
	visited = false;
	printed = false;
	Type = Undef;
	Name = "";
	State = "";
	Description = "";
	Function = "";
	Command = "";
	Example = "";
	fby.setAutoDelete( TRUE );
	fls.setAutoDelete( TRUE );
	typical.setAutoDelete( TRUE );
}
//---------------------------------------------------------------------------


/** No descriptions */
void Block::addTypicalFollowedby( Successor &s ){
	//std::cout << ID << " typically followed by " << s.ID << std::endl;
	Successor* sux = new Successor();
	sux->ID = s.ID;
	typical.append( sux );
}
//---------------------------------------------------------------------------


/** No descriptions */
void Block::addFollowedby( Successor &s ){
	//std::cout << ID << " followed by " << s.ID << std::endl;
	Successor* sux = new Successor();
	sux->ID = s.ID;
	fby.append( sux );
}
//---------------------------------------------------------------------------


/** No descriptions */
void Block::addFollows( Successor &s ){
	//std::cout << ID << " follows by " << s.ID << std::endl;
	Successor* sux = new Successor();
	sux->ID = s.ID;
	fls.append( sux );
}
//---------------------------------------------------------------------------


SuccessorList Block::getFollows(){
	return fls;
}
//---------------------------------------------------------------------------


SuccessorList Block::getFollowedby(){
	return fby;
}
//---------------------------------------------------------------------------


SuccessorList Block::getTypicalFollowedby(){
	return typical;
}
//---------------------------------------------------------------------------


