/***************************************************************************
                          prototype.cpp  -  description
                             -------------------
    begin                : Thu May 15 22:50:15 CEST 2003
    copyright            : (C) 2003 by Marcus Beranek
    email                : marcus@beranek.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qaccel.h>
#include <iostream>
#include <iomanip>
#include <qpainter.h>
#include <qprinter.h>
#include <qtextedit.h>
#include <qpaintdevicemetrics.h>
#include <qstringlist.h>
#include <qfile.h>

#include "prototype.h"
#include "filesave.xpm"
#include "fileopen.xpm"
#include "filenew.xpm"
#include "fullprotocol.xpm"
#include "treeview.xpm"
#include "settings.h"
#include "pluginaction.h"

/**
Main Class: creates the doc and view for prototype, reads and wirtes config, lists all protocols and plugins ...
*/
ProtoTypeApp::ProtoTypeApp()
{
	setCaption(tr("ProtoType " VERSION));

	readConfig();
	findPluginLibs();

	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initActions();
	initMenuBar();
	initToolBar();
	initStatusBar();

	initDoc();
	initView();

	view->setDoc( doc );

	connect( view, SIGNAL( protocolSelected( QString ) ), doc, SLOT( loadProtocol( QString ) ) );

	viewToolBar->setOn(true);
	viewStatusBar->setOn(true);

	doc->setConfigPrototypePath( config_protocol_path );
	doc->setConfigMaxCharactersPerLine( config_max_characters_per_line );
	doc->setConfigLanguage( config_language );

	readProtocols();

	// TODO: read from config...
	resize( 800, 600 );

	settings = new Settings( this );
	settings->hide();

	QString mcpl;
	mcpl.setNum( config_max_characters_per_line );

	//settings->setProtocolPath( config_protocol_path );
	//settings->setLanguage( config_language );
	//settings->setMaximumCharactersPerLine( mcpl );

	connect( settings, SIGNAL( configProtocolPathChanged( QString ) ), this, SLOT( setConfigProtocolPath( QString) ) );
	connect( settings, SIGNAL( configLanguageChanged( QString ) ), this, SLOT( setConfigLanguage(QString) )) ;
	connect( settings, SIGNAL( configMCPLChanged( QString ) ), this, SLOT( setConfigMCPL(QString) )) ; // MCPL = Max Characters per Line
	connect( settings, SIGNAL( configPluginDirChanged( QString ) ), this, SLOT( setConfigPluginDir(QString) )) ;


}
//---------------------------------------------------------------------------


ProtoTypeApp::~ProtoTypeApp()
{
}
//---------------------------------------------------------------------------


/** initializes all QActions of the application */
void ProtoTypeApp::initActions(){

	QPixmap openIcon, saveIcon, newIcon, fullIcon, treeView;
	newIcon = QPixmap(filenew);
	openIcon = QPixmap(fileopen);
	saveIcon = QPixmap(filesave);
	fullIcon = QPixmap(fullprotocol);
	treeView = QPixmap(treeview);


	fileNew = new QAction(tr("New File"), newIcon, tr("&New"), QAccel::stringToKey(tr("Ctrl+N")), this);
	fileNew->setStatusTip(tr("Creates a new document"));
	fileNew->setWhatsThis(tr("New File\n\nCreates a new document"));
	connect(fileNew, SIGNAL(activated()), this, SLOT(slotFileNew()));

	fileOpen = new QAction(tr("Open File"), openIcon, tr("&Open..."), 0, this);
	fileOpen->setStatusTip(tr("Opens an existing document"));
	fileOpen->setWhatsThis(tr("Open File\n\nOpens an existing document"));
	connect(fileOpen, SIGNAL(activated()), this, SLOT(slotFileOpen()));

	fileSave = new QAction(tr("Save File"), saveIcon, tr("&Save"), QAccel::stringToKey(tr("Ctrl+S")), this);
	fileSave->setStatusTip(tr("Saves the actual document"));
	fileSave->setWhatsThis(tr("Save File.\n\nSaves the actual document"));
	connect(fileSave, SIGNAL(activated()), this, SLOT(slotFileSave()));

	fileSaveAs = new QAction(tr("Save File As"), tr("Save &as..."), 0, this);
	fileSaveAs->setStatusTip(tr("Saves the actual document under a new filename"));
	fileSaveAs->setWhatsThis(tr("Save As\n\nSaves the actual document under a new filename"));
	connect(fileSaveAs, SIGNAL(activated()), this, SLOT(slotFileSave()));

	fileClose = new QAction(tr("Close File"), tr("&Close"), QAccel::stringToKey(tr("Ctrl+W")), this);
	fileClose->setStatusTip(tr("Closes the actual document"));
	fileClose->setWhatsThis(tr("Close File\n\nCloses the actual document"));
	connect(fileClose, SIGNAL(activated()), this, SLOT(slotFileClose()));

	filePrint = new QAction(tr("Print File"), tr("&Print"), QAccel::stringToKey(tr("Ctrl+P")), this);
	filePrint->setStatusTip(tr("Prints out the actual document"));
	filePrint->setWhatsThis(tr("Print File\n\nPrints out the actual document"));
	connect(filePrint, SIGNAL(activated()), this, SLOT(slotFilePrint()));

	fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
	fileQuit->setStatusTip(tr("Quits the application"));
	fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
	connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

	editCut = new QAction(tr("Cut"), tr("Cu&t"), QAccel::stringToKey(tr("Ctrl+X")), this);
	editCut->setStatusTip(tr("Cuts the selected section and puts it to the clipboard"));
	editCut->setWhatsThis(tr("Cut\n\nCuts the selected section and puts it to the clipboard"));
	connect(editCut, SIGNAL(activated()), this, SLOT(slotEditCut()));

	editCopy = new QAction(tr("Copy"), tr("&Copy"), QAccel::stringToKey(tr("Ctrl+C")), this);
	editCopy->setStatusTip(tr("Copies the selected section to the clipboard"));
	editCopy->setWhatsThis(tr("Copy\n\nCopies the selected section to the clipboard"));
	connect(editCopy, SIGNAL(activated()), this, SLOT(slotEditCopy()));

	editPaste = new QAction(tr("Paste"), tr("&Paste"), QAccel::stringToKey(tr("Ctrl+V")), this);
	editPaste->setStatusTip(tr("Pastes the clipboard contents to actual position"));
	editPaste->setWhatsThis(tr("Paste\n\nPastes the clipboard contents to actual position"));
	connect(editPaste, SIGNAL(activated()), this, SLOT(slotEditPaste()));

	viewToolBar = new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
	viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
	viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
	connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));

	viewStatusBar = new QAction(tr("Statusbar"), tr("&Statusbar"), 0, this, 0, true);
	viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
	viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
	connect(viewStatusBar, SIGNAL(toggled(bool)), this, SLOT(slotViewStatusBar(bool)));

	viewCompleteProtocol = new QAction(tr("Typical Protocol"), fullIcon, tr("&Typical Protocol"), 0, this, 0, true);
	viewCompleteProtocol->setStatusTip(tr("toggles view of complete or typical protocol"));
	viewCompleteProtocol->setWhatsThis(tr("Typical Protocol\n\nIf toggled, shows only the typical protocol. Otherwise shows the complete protocol."));
	connect(viewCompleteProtocol, SIGNAL(toggled(bool)), this, SLOT(slotCompleteProtocol(bool)));

	viewSettings = new QAction(tr("Preferences"), tr("Preferences"), QAccel::stringToKey(tr("Ctrl+O")), this);
	viewSettings->setStatusTip(tr("View/edit the program-settings"));
	viewSettings->setWhatsThis(tr("View/edit the program-settings."));
	connect(viewSettings, SIGNAL( activated() ), this, SLOT(slotSettings() ));


	helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
	helpAboutApp->setStatusTip(tr("About the application"));
	helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
	connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

}
//---------------------------------------------------------------------------


void ProtoTypeApp::initMenuBar()
{
	///////////////////////////////////////////////////////////////////
	// MENUBAR

	///////////////////////////////////////////////////////////////////
	// menuBar entry fileMenu
	fileMenu=new QPopupMenu();
	//  fileNew->addTo(fileMenu);
	//  fileOpen->addTo(fileMenu);
	//  fileClose->addTo(fileMenu);
	//  fileMenu->insertSeparator();
	//  fileSave->addTo(fileMenu);
	//  fileSaveAs->addTo(fileMenu);
	//  fileMenu->insertSeparator();
	filePrint->addTo(fileMenu);
	fileMenu->insertSeparator();
	fileQuit->addTo(fileMenu);

	///////////////////////////////////////////////////////////////////
	// menuBar entry editMenu
	editMenu=new QPopupMenu();
	editCut->addTo(editMenu);
	editCopy->addTo(editMenu);
	editPaste->addTo(editMenu);
	viewSettings->addTo(editMenu);

	///////////////////////////////////////////////////////////////////
	// menuBar entry viewMenu
	viewMenu=new QPopupMenu();
	viewMenu->setCheckable(true);
	viewToolBar->addTo(viewMenu);
	viewStatusBar->addTo(viewMenu);
	viewCompleteProtocol->addTo(viewMenu);

	///////////////////////////////////////////////////////////////////
	// EDIT YOUR APPLICATION SPECIFIC MENUENTRIES HERE

	// Plugin-Menu...
	//
	// Usually we connect a QAction to one specific slot. But because we do not know
	// how many plugins we have, we cannont provide a separate slot for every plugin.
	//
	// Workaround:
	// Each plugin-library, which we have stored in the pluginlist, gets a PluginAction.
	// A PluginAction is inherited from QAction and provides an additional signal,
	// which emits the library-name, when activated. So we can use this signal to determine, 
	// which plugin was selected by just using a single slot.
	//
	pluginMenu = new QPopupMenu();
	for ( QStringList::Iterator it = pluginlist.begin(); it != pluginlist.end(); ++it ){
		std::cout << "Creating menu for " << *it << endl;
		QString libName = *it;
		QString plugName = *it;
		plugName = plugName.remove( 0, 12 );
		plugName = plugName.remove( plugName.length()-3 ,3 );
		PluginAction* plugAction = new PluginAction( plugName, plugName, QAccel::stringToKey(tr("")), this);
		plugAction->setLibname( libName );
		plugAction->setStatusTip(tr("load  " + plugName + "-plugin."));
		plugAction->setWhatsThis(tr("Loads the " + plugName + "-plugin."));
		plugAction->addTo( pluginMenu );
		connect( plugAction, SIGNAL( pluginActivated( QString ) ), this, SLOT( slotPluginSelected( QString ) ) );
	}

	///////////////////////////////////////////////////////////////////
	// menuBar entry helpMenu
	helpMenu=new QPopupMenu();
	helpAboutApp->addTo(helpMenu);

	///////////////////////////////////////////////////////////////////
	// MENUBAR CONFIGURATION
	menuBar()->insertItem(tr("&File"), fileMenu);
	menuBar()->insertItem(tr("&Edit"), editMenu);
	menuBar()->insertItem(tr("&View"), viewMenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem(tr("Pl&ugins"), pluginMenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem(tr("&Help"), helpMenu);

}
//---------------------------------------------------------------------------


void ProtoTypeApp::initToolBar()
{
	///////////////////////////////////////////////////////////////////
	// TOOLBAR
	fileToolbar = new QToolBar(this, "file operations");
	//
	// no need for this now:
	//
	//  fileNew->addTo(fileToolbar);
	//  fileOpen->addTo(fileToolbar);
	//  fileSave->addTo(fileToolbar);
	//  fileToolbar->addSeparator();
	viewCompleteProtocol->addTo(fileToolbar);

	fileToolbar->addSeparator();
	QWhatsThis::whatsThisButton(fileToolbar);

}
//---------------------------------------------------------------------------


void ProtoTypeApp::initStatusBar()
{
	///////////////////////////////////////////////////////////////////
	//STATUSBAR
	statusBar()->message(tr("Ready."), 2000);
}
//---------------------------------------------------------------------------


void ProtoTypeApp::initDoc()
{
	doc=new ProtoTypeDoc();
}
//---------------------------------------------------------------------------


void ProtoTypeApp::initView()
{
	////////////////////////////////////////////////////////////////////
	// set the main widget here
	view=new ProtoTypeView(this, doc);
	setCentralWidget(view);
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotPluginSelected( QString pl )
{
	//
	// if a plugin is selected from the plugin-menu, we get a signal from the PluginAction.
	// So we just have to tell the view-object, to load this plugin.
	//
	if(view){
		view->launchPlugin( pl );
	}
}
//---------------------------------------------------------------------------


bool ProtoTypeApp::queryExit()
{
	int exit=QMessageBox::information(this, tr("Quit..."),
	                                  tr("Do your really want to quit?"),
	                                  QMessageBox::Ok, QMessageBox::Cancel);

	if (exit==1)
	{

	}
	else
	{

	};

	return (exit==1);
}
//---------------------------------------------------------------------------


/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void ProtoTypeApp::slotFileNew()
{
	statusBar()->message(tr("Creating new file..."));
	doc->newDoc();
	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotFileOpen()
{
	statusBar()->message(tr("Opening file..."));

	QString fileName = QFileDialog::getOpenFileName(0,0,this);
	if (!fileName.isEmpty())
	{
		doc->load(fileName);
		setCaption(fileName);
		QString message=tr("Loaded document: ")+fileName;
		statusBar()->message(message, 2000);
	}
	else
	{
		statusBar()->message(tr("Opening aborted"), 2000);
	}
}
//---------------------------------------------------------------------------



void ProtoTypeApp::slotFileSave()
{
	statusBar()->message(tr("Saving file..."));
	doc->save();
	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotFileSaveAs()
{
	statusBar()->message(tr("Saving file under new filename..."));
	QString fn = QFileDialog::getSaveFileName(0, 0, this);
	if (!fn.isEmpty())
	{
		doc->saveAs(fn);
	}
	else
	{
		statusBar()->message(tr("Saving aborted"), 2000);
	}

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotFileClose()
{
	statusBar()->message(tr("Closing file..."));

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotFilePrint()
{
	const int Margin = 10;
	int pageNo = 1;

	statusBar()->message(tr("Printing..."));
	QPrinter printer;
	if (printer.setup(this))
	{
		QPainter painter;
		painter.begin(&printer);

		//
		// Most parts of the printing-code is taken from the QT-Documentation
		// of the qpainter-device
		//

		QTextEdit* e = view->getTextEdit();
		QString n;
		n.append( view->getCurrentProtocol() );
		n.append( ": ");
		n.append( view->getCurrentBlockName() );

		painter.setFont( e->font() );
		int yPos        = 0;                    // y-position for each line
		QFontMetrics fm = painter.fontMetrics();
		QPaintDeviceMetrics metrics( &printer ); // need width/height of printer surface
		
		for( int i = 0 ; i < e->lines() ; i++ ) {
			if ( Margin + yPos > metrics.height() - Margin ) {
				QString msg( "Printing (page " );
				msg += QString::number( ++pageNo );
				msg += ")...";
				statusBar()->message( msg );
				printer.newPage();              // no more room on this page
				yPos = 0;                       // back to top of page
			}

			if( yPos == 0 ){
				// Draw a HEADLINE...
				painter.drawText(Margin, Margin + yPos,
				                 metrics.width(), fm.lineSpacing(),
				                 ExpandTabs | DontClip,
				                 n);
				yPos = yPos + fm.lineSpacing();

				painter.drawLine( Margin, yPos+fm.lineSpacing(), metrics.width(), yPos+fm.lineSpacing() );
				yPos = yPos + fm.lineSpacing();
			}

			painter.drawText( Margin, Margin + yPos,
			                  metrics.width(), fm.lineSpacing(),
			                  ExpandTabs | DontClip,
			                  e->text( i ) );
			yPos = yPos + fm.lineSpacing();

		}


		painter.end();
	};

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotFileQuit()
{
	statusBar()->message(tr("Exiting application..."));
	///////////////////////////////////////////////////////////////////
	// exits the Application
	if(doc->isModified())
	{
		if(queryExit())
		{
			qApp->quit();
		}
		else
		{

		};
	}
	else
	{
		qApp->quit();
	};

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


// TODO: Clipboard-Stuff...
void ProtoTypeApp::slotEditCut()
{
	statusBar()->message(tr("Cutting selection..."));

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------

// TODO: Clipboard-Stuff...
void ProtoTypeApp::slotEditCopy()
{
	statusBar()->message(tr("Copying selection to clipboard..."));

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------

// TODO: Clipboard-Stuff...
void ProtoTypeApp::slotEditPaste()
{
	statusBar()->message(tr("Inserting clipboard contents..."));

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotViewToolBar(bool toggle)
{
	statusBar()->message(tr("Toggle toolbar..."));
	///////////////////////////////////////////////////////////////////
	// turn Toolbar on or off

	if (toggle== false)
	{
		fileToolbar->hide();
	}
	else
	{
		fileToolbar->show();
	};

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotViewStatusBar(bool toggle)
{
	statusBar()->message(tr("Toggle statusbar..."));
	///////////////////////////////////////////////////////////////////
	//turn Statusbar on or off

	if (toggle == false)
	{
		statusBar()->hide();
	}
	else
	{
		statusBar()->show();
	}

	statusBar()->message(tr("Ready."));
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotCompleteProtocol(bool s)
{
	//
	// We can choose between full and typical protocol.
	// So he vae to notify the view of that decision.
	//
	if (s) view->setShowState( ProtoTypeView::ShowTypical );
	else view->setShowState( ProtoTypeView::ShowAll );
}
//---------------------------------------------------------------------------


void ProtoTypeApp::slotHelpAbout()
{
	QMessageBox::about(this,tr("About..."),
	                   tr("ProtoType\nVersion " VERSION "\n(c) 2003 by Marcus Beranek") );
}
//---------------------------------------------------------------------------

void ProtoTypeApp::readConfig()
{
	//
	// The config-file is stored in the users homedir and called .prototyperc
	// For now it is nothing more than a key=value list.
	//

	QStringList lines;
	QDir d;
	QFile file( d.homeDirPath()+"/.prototyperc" );

	if ( file.open( IO_ReadOnly ) ) {
		
		// Config-File exists:
		QTextStream stream( &file );
		QString line;
		
		while ( !stream.eof() ) {
			line = stream.readLine();
			if( line.startsWith( "protocol_path=" ) ){
				config_protocol_path = line.remove( 0, 14 );
			}
			if( line.startsWith( "max_characters_per_line=" ) ){
				config_max_characters_per_line = line.remove( 0, 24 ).toInt();
			}
			if( line.startsWith( "language=" ) ){
				config_language = line.remove( 0, 9 );
			}
			if( line.startsWith( "plugindir=" ) ){
				config_plugindir = line.remove( 0, 10 );
			}

		}
		
		file.close();
		
	} else {
		
		// No Config-File exists, 
		// so we create one:
		
		// set default-values:
		config_protocol_path = "/usr/local/share/prototype/protocols";
		config_max_characters_per_line = 100;
		config_language="en";
		config_plugindir="/usr/local/lib";
		
		// write file:
		writeConfigFile();
		
		// and notify user, that we have created the config-file...
		QMessageBox::information(this, tr("Config-File created..."), tr("Created config-file: ") + d.homeDirPath()+"/.prototyperc" );

	}
}
//---------------------------------------------------------------------------


void ProtoTypeApp::readProtocols()
{

	// Protocol-files are living in a separate directory. The directory-name
	// is stored in the config-file

	// TODO: In der ListBox (in view) sind im Augenblick nur die Dateinamen (ohne .xml) abgelegt.
	// Hier sollten zur besseren bersicht die Protokollnamen stehen.
	// Es muss also noch eine Liste ProtokollName <-> Dateiname angelegt werden.

	QDir d;
	d.setPath( config_protocol_path + "/" + config_language );
	std::cout << "[readProtocols] " << config_protocol_path + "/" + config_language << std::endl;
	d.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks );
	d.setNameFilter( "*.xml" );

	const QFileInfoList *list = d.entryInfoList();
	if (list){
		QFileInfoListIterator it( *list );
		QFileInfo *fi;

		view->clearProtocolList();

		while ( (fi = it.current()) != 0 ) {

			view->addProtocol( fi->fileName() );
			++it;
		}
	}

}
//---------------------------------------------------------------------------


// TODO: find out, what's the difference between: "options", "settings", "preferences", ...
void ProtoTypeApp::slotSettings()
{
	//
	// show actual values in the settings-dialog.
	//
	QString mcpl;
	mcpl.setNum( config_max_characters_per_line );
	settings->setLanguage( config_language );
	settings->setMaximumCharactersPerLine( mcpl );
	settings->setProtocolPath( config_protocol_path );
	settings->setPluginDir( config_plugindir );
	settings->show();
}
//---------------------------------------------------------------------------


void ProtoTypeApp::setConfigProtocolPath( QString p )
{
	config_protocol_path = p;
	writeConfigFile();
	doc->setConfigPrototypePath( config_protocol_path );
	doc->setConfigLanguage( config_language );
	readProtocols();
	doc->loadProtocol( doc->getCurrentProtocol() + ".xml" );
}
//---------------------------------------------------------------------------


void ProtoTypeApp::setConfigLanguage( QString l )
{
	config_language = l;
	writeConfigFile();
	doc->setConfigPrototypePath( config_protocol_path );
	doc->setConfigLanguage( config_language );
	readProtocols();
	doc->loadProtocol( doc->getCurrentProtocol() + ".xml" );
}
//---------------------------------------------------------------------------


void ProtoTypeApp::setConfigMCPL(QString mcpl )
{
	config_max_characters_per_line = mcpl.toInt();
	writeConfigFile();
	doc->setConfigMaxCharactersPerLine( config_max_characters_per_line );
	doc->loadProtocol( doc->getCurrentProtocol() + ".xml" );
}
//---------------------------------------------------------------------------


void ProtoTypeApp::setConfigPluginDir(QString pd )
{
	config_plugindir = pd;
	writeConfigFile();
	
	QMessageBox::information(this, tr("Plugin-Directory changed..."),
	                                  tr("Your plugin-directory has changed.\nChanges will take effect the next time you start prototype.\nPlease make sure that the directory\n\n  ") + pd + tr("\n\nis included in your /etc/ld.so.conf and that you\nhave run \"ldconfig\" as root.") );
}
//---------------------------------------------------------------------------


void ProtoTypeApp::writeConfigFile()
{
	//
	// Store the configuration in the users homedir as a key=value-file.
	//
	
	QStringList lines;
	QDir d;
	QFile file( d.homeDirPath()+"/.prototyperc" );

	QString mcpl;
	mcpl.setNum(config_max_characters_per_line);

	if ( file.open( IO_WriteOnly ) ) {
		
		QTextStream stream( &file );
		
		lines.append("protocol_path=" + config_protocol_path );
		lines.append("max_characters_per_line=" + mcpl );
		lines.append("language=" + config_language );
		lines.append("plugindir=" + config_plugindir );
		
		for ( QStringList::Iterator it = lines.begin(); it != lines.end(); ++it )
			stream << *it << "\n";
		file.close();
	}
}
//---------------------------------------------------------------------------


void ProtoTypeApp::findPluginLibs()
{
	QStringList stddirs;
	QDir d;
	
	//
	// Search plugindir from config-file and some standard-directories...
	//
	
	stddirs.append( config_plugindir );
	if( config_plugindir != "/lib" ) stddirs.append("/lib");
	if( config_plugindir != "/usr/lib" ) stddirs.append("/usr/lib");
	if( config_plugindir != "/usr/local/lib" ) stddirs.append("/usr/local/lib");
	if( config_plugindir != "/opt/lib" ) stddirs.append("/opt/lib");
	if( config_plugindir != "/usr/local/kde/lib" ) stddirs.append("/usr/local/kde/lib");
	if( config_plugindir != "/usr/local/qt/lib" ) stddirs.append("/usr/local/qt/lib");
	if( config_plugindir != "/usr/local/share/prototype/plugins" ) stddirs.append("/usr/local/share/prototype/plugins");

	for ( QStringList::Iterator it = stddirs.begin(); it != stddirs.end(); ++it ){
		std::cout << "searching directory: " << *it << endl;
		
		d.setPath( *it );
		d.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks );
		d.setNameFilter( "libprototype*.so" );
		
		const QFileInfoList *list = d.entryInfoList();
		if (list){
			QFileInfoListIterator it2( *list );
			QFileInfo *fi;
			while ( (fi = it2.current()) != 0 ) {
				std::cout << "Found plugin: " << fi->fileName() << endl;
				pluginlist.append( fi->fileName() );
				++it2;
			}
		}
	}
}
//---------------------------------------------------------------------------




